.jetpack-recipe-meta li.jetpack-recipe-print {
	display: none;
}

.jetpack-recipe-title {
	font-size: 16pt;
}

.jetpack-recipe-content img {
	display: inline-block !important;
	max-width: 100%;
}

.jetpack-recipe-image {
	display: none !important;
}

.jetpack-recipe-content .aligncenter {
	display: block !important;
	margin: 0 auto 1em !important;
	text-align: center !important;
}

.jetpack-recipe-content .alignright {
	float: right !important;
	margin: 0 0 .5em 1em !important;
}

.jetpack-recipe-content .alignleft {
	float: left !important;
	margin: 0 1em .5em 0 !important;
}

.jetpack-recipe-content .alignnone {
	display: inline-block;
}
