/**
 * Styles for the [gravatar_profile] shortcode when it is called from an AMP context
 */
 .grofile-wrap {
	border: solid 1px #f0f0f1;
	padding: 10px;
}

.grofile {
	padding: 0 0 5px 0;
}

.grofile-left {
	float: left;
	display: block;
	width: 96px;
	margin-right: 15px;
}

.grofile .gravatar {
	margin-bottom: 5px;
}

.grofile-clear {
	clear: left;
	font-size: 1px;
	height: 1px;
}

.grofile ul li a {
	text-indent: -99999px;
}

.grofile .grofile-left a:hover {
	text-decoration: none !important;
	border: none !important;
}

.grofile-name {
	margin-top: 0;
}
